@@ -147,7 +147,7 @@ def __test_relation_bert(test_file_path, tokenizer_folder_path, model_folder_pat
 
 def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_file_path, rel_tokenizer_folder_path,
                   rel_model_folder_path, quid_match_path, output_folder_path, parallel_print_files, export_text,
-                  export_ref, output_type, csv_sep):
+                  export_ref, output_type, csv_sep, open_quote, close_quote):
 
     reference_vectorizer = ReferenceVectorizer.from_vocab_file(ref_vocab_file_path, 25, True)
     reference_model_trainer = ReferenceModelTrainer(25, True, 32, 32, 0.2, 512, 10)
@@ -160,7 +160,8 @@ def __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_
     with open(source_file_path, 'r', encoding='utf-8') as source_file:
         source_file_content = source_file.read().lower()
 
-    pro_quo = ProQuo(reference_model, reference_vectorizer, relation_model, relation_vectorizer)
+    pro_quo = ProQuo(reference_model, reference_vectorizer, relation_model, relation_vectorizer, open_quote,
+                     close_quote)
 
     if isfile(target_path) and target_path.endswith('.txt'):
         with open(target_path, 'r', encoding='utf-8') as target_file:
@@ -338,7 +339,7 @@ def main(argv=None):
     parser_compare.add_argument('--no-create-dated-subfolder', dest='create_dated_subfolder',
                                 action='store_false',
                                 help='Do not create a subfolder named with the current date to store the results')
-    parser_compare.add_argument('--parallel-print-files', dest='parallel_print_files', nargs='*')
+    parser_compare.add_argument('--parallel-print-files', dest='parallel_print_files', nargs='*', default=[])
     parser_compare.add_argument('--text', dest="export_text", default=True, action='store_true',
                                 help="Include matched text in the returned data structure")
     parser_compare.add_argument('--no-text', dest='export_text', action='store_false',
@@ -351,6 +352,12 @@ def main(argv=None):
                                 help="The output type")
     parser_compare.add_argument('--csv-sep', dest='csv_sep', type=str, help="output separator for csv (default: '\\t')",
                                 default='\t')
+    parser_compare.add_argument('--open-quote', dest='open_quote', type=str, help="The quotation open character"
+                                                                                  " (default: '\"')",
+                                default='"')
+    parser_compare.add_argument('--close-quote', dest='close_quote', type=str, help="The quotation close character"
+                                                                                    " (default: '\"')",
+                                default='"')
 
     args = argument_parser.parse_args(argv)
 
@@ -423,6 +430,8 @@ def main(argv=None):
         export_ref = args.export_ref
         output_type = args.output_type
         csv_sep = bytes(args.csv_sep, "utf-8").decode("unicode_escape")
+        open_quote = args.open_quote
+        close_quote = args.close_quote
 
         if create_dated_subfolder:
             now = datetime.now()
@@ -432,7 +441,7 @@ def main(argv=None):
 
         __run_compare(source_file_path, target_path, ref_vocab_file_path, ref_model_file_path,
                       rel_tokenizer_folder_path, rel_model_folder_path, quid_match_path, output_folder_path,
-                      parallel_print_files, export_text, export_ref, output_type, csv_sep)
+                      parallel_print_files, export_text, export_ref, output_type, csv_sep, open_quote, close_quote)
 
 
 if __name__ == '__main__':
