@@ -5,19 +5,19 @@ from typing import Optional, List, Tuple
 from quid.match.Match import Match
 from quid.match.MatchSpan import MatchSpan
 
+from proquo.core import Helper
 from proquo.match.MatchRef import MatchRef
 from proquo.core.Quote import Quote
 from proquo.core.QuoteRef import QuoteRef
 from proquo.core.Reference import Reference
 import numpy as np
 from sklearn.cluster import SpectralClustering
-from datasketch import MinHash, MinHashLSH
-from rapidfuzz.distance import Levenshtein
 import tensorflow as tf
 
 
 # noinspection PyMethodMayBeStatic
 class ProQuo:
+    # Todo: Make these customizable
     # References
     MAX_BRACKET_LENGTH = 25
     LONG_MIN_LENGTH = 5
@@ -31,11 +31,16 @@ class ProQuo:
     WITHOUT_REF_SEARCH_RADIUS = 500
     SOURCE_PARALLEL_LAST_PAGE = 63
     # Fuzzy matching
-    HASH_PERM: int = 128
-    LSH_THRESHOLD: float = 0.70
     SCORE_CUTOFF: int = 0.85
 
-    def __init__(self, reference_model, reference_vectorizer, relation_model, relation_vectorizer):
+    def __init__(self, reference_model, reference_vectorizer, relation_model, relation_vectorizer, open_quote,
+                 close_quote):
+        """
+        :param reference_model:
+        :param reference_vectorizer:
+        :param relation_model:
+        :param relation_vectorizer:
+        """
         self.reference_model = reference_model
         self.reference_vectorizer = reference_vectorizer
         self.relation_model = relation_model
@@ -43,21 +48,34 @@ class ProQuo:
         self.source_cache = {}
         self.hashes = None
         self.source_text_parallel_print = False
+        self.open_quote = open_quote
+        self.close_quote = close_quote
 
-    def compare(self, source_text: str, target_text: str, quidl_matches: List[Match],
+    def compare(self, source_text: str, target_text: str, quid_matches: List[Match],
                 source_text_parallel_print=False) -> List[MatchRef]:
-        source_text = self.__clean_text(source_text)
+        """
+        Compare the two input texts and return a list of matching sequences.
+        :param source_text: A source text
+        :param target_text: A target text
+        :param quid_matches: A list of matches from Quid to be used as candidates for short matches
+        :param source_text_parallel_print:
+        :return: A list of found matches
+        """
+
+        source_text = Helper.clean_text(source_text)
+
+        # Check if text was already processed and use cached version for speed up
         source_text_hash = hash(source_text)
         if source_text_hash in self.source_cache:
             self.hashes = self.source_cache[source_text_hash]
         else:
-            self.hashes = self.__init_lsh_hashes(source_text)
+            self.hashes = Helper.init_lsh_hashes(source_text)
             self.source_cache[source_text_hash] = self.hashes
 
         self.source_text_parallel_print = source_text_parallel_print
 
-        long_matches = self.__filter_long_matches(source_text, quidl_matches)
-        short_matches = self.__filter_short_matches(source_text, target_text, quidl_matches)
+        long_matches = self.__filter_long_matches(source_text, quid_matches)
+        short_matches = Helper.filter_short_matches(source_text, target_text, quid_matches, self.LONG_MIN_LENGTH)
 
         matches_with_reference: List[MatchRef] = self.__add_reference_to_match(target_text, long_matches)
         matches_with_reference.sort(key=lambda x: x.target_span.end - x.target_span.start, reverse=True)
@@ -72,26 +90,21 @@ class ProQuo:
                 break
 
         all_references: List[Reference] = self.__get_references(target_text, longest_matches)
-        all_quotes: List[Quote] = self.__get_quotations(target_text)
-        footnote_ranges = self.__get_footnote_ranges(target_text)
+        all_quotes: List[Quote] = Helper.get_quotations(target_text, self.LONG_MIN_LENGTH, self.open_quote,
+                                                        self.close_quote)
+        footnote_ranges = Helper.get_footnote_ranges(target_text)
 
         main_references = []
-        footnote_references = []
 
         for ref in all_references:
-            if not self.__is_in_footnote(ref.start, ref.end, footnote_ranges):
+            if not Helper.is_in_footnote(ref.start, ref.end, footnote_ranges):
                 main_references.append(ref)
-            else:
-                footnote_references.append(ref)
 
         main_quotes = []
-        footnote_quotes = []
 
         for q in all_quotes:
-            if not self.__is_in_footnote(q.start, q.end, footnote_ranges):
+            if not Helper.is_in_footnote(q.start, q.end, footnote_ranges):
                 main_quotes.append(q)
-            else:
-                footnote_quotes.append(q)
 
         main_quotes = self.__filter_quotes_not_in_source(main_quotes, 0, len(source_text))
 
@@ -194,7 +207,7 @@ class ProQuo:
 
         for qrc in quote_ref_combos:
             text = qrc.text
-            text = re.sub(f'"', ' " ', text, flags=re.DOTALL)
+            text = re.sub(fr'([{self.open_quote}{self.close_quote}])', r' \1 ', text, flags=re.DOTALL)
             sentences.append(text)
 
         test_data = self.relation_vectorizer.vectorize(sentences)
@@ -217,19 +230,6 @@ class ProQuo:
 
         return result
 
-    def __filter_short_matches(self, source_text, target_text, matches):
-        result = []
-        for match in matches:
-            match_source_text = source_text[match.source_span.start:match.source_span.end]
-            match_target_text = target_text[match.target_span.start:match.target_span.end]
-            source_length = len(match_source_text.split())
-            target_length = len(match_target_text.split())
-
-            if source_length < self.LONG_MIN_LENGTH or target_length < self.LONG_MIN_LENGTH:
-                result.append(match)
-
-        return result
-
     def __add_reference_to_match(self, input_text, matches) -> List[MatchRef]:
         result: List[MatchRef] = []
 
@@ -346,19 +346,6 @@ class ProQuo:
 
         return references
 
-    def __get_quotations(self, input_text: str) -> List[Quote]:
-        quotes = []
-
-        for re_match in re.finditer(r'\"([^"]+?)\"', input_text):
-            start = re_match.span(1)[0]
-            end = re_match.span(1)[1]
-            text = re_match.group(1)
-
-            if len(text.split()) < self.LONG_MIN_LENGTH:
-                quotes.append(Quote(start, end, text))
-
-        return quotes
-
     def __replace_other_ref(self, input_text, references, start, end):
         other_refs = []
 
@@ -401,7 +388,7 @@ class ProQuo:
                     continue
 
                 text_replaced = self.__replace_other_ref(input_text, references, quote.end, reference.start)
-                text = f' <Q> {input_text[quote.start:quote.end]} </Q> {text_replaced} <REF> '
+                text = f' <Q> {input_text[quote.start:quote.end]} </Q> {text_replaced[1:]} <REF> '
             else:
                 start = reference.start - offset
                 end = quote.end - offset
@@ -417,7 +404,7 @@ class ProQuo:
                     continue
 
                 text_replaced = self.__replace_other_ref(input_text, references, reference.end, quote.start)
-                text = f' <REF> {text_replaced} <Q> {input_text[quote.start:quote.end]} </Q> '
+                text = f' <REF> {text_replaced[:-1]} <Q> {input_text[quote.start:quote.end]} </Q> '
 
             length = len(text.split())
 
@@ -443,7 +430,7 @@ class ProQuo:
                 part_len = len(part)
                 temp_start = temp_end - 1 - part_len
 
-                if not footnote_ranges or not self.__is_in_footnote(temp_start, temp_end, footnote_ranges):
+                if not footnote_ranges or not Helper.is_in_footnote(temp_start, temp_end, footnote_ranges):
                     text_before = f'{parts_before[i]} {text_before}'
 
                 temp_end = temp_start
@@ -456,7 +443,7 @@ class ProQuo:
                 part_len = len(part)
                 temp_end = temp_start + 1 + part_len
 
-                if not footnote_ranges or not self.__is_in_footnote(temp_start, temp_end, footnote_ranges):
+                if not footnote_ranges or not Helper.is_in_footnote(temp_start, temp_end, footnote_ranges):
                     text_after += f' {parts_after[i]}'
 
                 temp_start = temp_end
@@ -464,34 +451,15 @@ class ProQuo:
             text_before_replaced = self.__replace_other_ref(input_text, references, start - len(text_before), start)
             text_after_replaced = self.__replace_other_ref(input_text, references, end, end + len(text_after))
 
-            text = f'{text_before_replaced} {text} {text_after_replaced}'
+            if quote.start < reference.end:
+                text = f'{text_before_replaced[:-1]} {text[1:]} {text_after_replaced}'
+            else:
+                text = f'{text_before_replaced} {text[:-1]} {text_after_replaced[1:]}'
 
             result.append(QuoteRef(quote, reference, text))
 
         return result
 
-    def __get_combinations_in_footnotes(self, input_text, quotes, references, footnote_ranges):
-        result = []
-
-        for fn_range in footnote_ranges:
-            fn_quotes = []
-            fn_references = []
-
-            for q in quotes:
-                if fn_range[0] <= q.start < fn_range[1]:
-                    fn_quotes.append(q)
-
-            for ref in references:
-                if fn_range[0] <= ref.start < fn_range[1]:
-                    fn_references.append(ref)
-
-            if len(fn_quotes) > 0 and len(fn_references) > 0:
-                text = input_text[fn_range[0]:fn_range[1]]
-                combos = self.__get_combinations(text, fn_references, fn_quotes, None, fn_range[0])
-                result.extend(combos)
-
-        return result
-
     def __get_short_quotes(self, quote_ref_combos) -> List[QuoteRef]:
         preds = self.__predict_rel(quote_ref_combos)
         result = []
@@ -511,7 +479,7 @@ class ProQuo:
                 result.append(quote)
                 continue
 
-            candidates = self.__fuzzy_match(quote.text, start, end)
+            candidates = Helper.fuzzy_match(quote.text, start, end, self.hashes, self.SCORE_CUTOFF)
             if len(candidates) == 0:
                 continue
 
@@ -583,23 +551,6 @@ class ProQuo:
 
         return 0
 
-    def __get_footnote_ranges(self, input_text):
-        result = []
-
-        for re_match in re.finditer(r'\[\[\[((?:.|\n)+?)]]]', input_text):
-            start = re_match.start()
-            end = re_match.end()
-            result.append((start, end))
-
-        return result
-
-    def __is_in_footnote(self, start, end, footnote_ranges):
-        for rf in footnote_ranges:
-            if (rf[0] <= start < rf[1]) or (rf[0] <= end <= rf[1]):
-                return True
-
-        return False
-
     def __count_words_in_footnotes(self, input_text, footnote_ranges, start, end):
         count = 0
 
@@ -610,94 +561,6 @@ class ProQuo:
 
         return count
 
-    def __clean_text(self, input_text: str) -> str:
-        output_text = re.sub('(\\[\\.\\.\\.]|\\[…]|\\.\\.\\.|…)', lambda x: ' ' * len(x.group(1)), input_text)
-        output_text = re.sub(f'[^a-zA-Z\\däüöÄÜÖß\n ]', ' ', output_text)
-
-        if len(input_text) != len(output_text):
-            raise Exception('Length of source text changed')
-
-        return output_text.lower()
-
-    def __normalize_special_chars(self, input_word: str) -> str:
-        input_word = input_word.replace('ß', 'ss')
-        input_word = input_word.replace('ä', 'ae')
-        input_word = input_word.replace('ö', 'oe')
-        input_word = input_word.replace('ü', 'ue')
-        input_word = input_word.replace('ey', 'ei')
-
-        input_word = input_word.replace('[', '')
-        input_word = input_word.replace(']', '')
-        return input_word
-
-    def __init_lsh_hashes(self, source_text) -> MinHashLSH:
-        hashes = MinHashLSH(threshold=self.LSH_THRESHOLD, num_perm=self.HASH_PERM)
-
-        for match in re.finditer(r'\S+', source_text):
-            token = self.__normalize_special_chars(match.group())
-
-            if len(token) > 0:
-                text_begin_pos = match.start()
-                text_end_pos = match.end()
-
-                token_character_set = set(token)
-                token_hash = MinHash(num_perm=self.HASH_PERM)
-
-                for char in token_character_set:
-                    token_hash.update(char.encode('utf8'))
-
-                hashes.insert(f'{text_begin_pos}_{text_end_pos}_{token}', token_hash, True)
-
-        return hashes
-
-    # TODO: improve with better normalization
-    def __strict_match(self, word: str, search_space: str):
-        word = self.__clean_text(word)
-        # word = self.__normalize_special_chars(word)
-        # word = re.sub(' +', ' ', word, flags=re.DOTALL)
-        # word = word.strip()
-        # search_space = self.__normalize_special_chars(search_space)
-
-        re_matches_iter = re.finditer(r'\b' + re.escape(word) + r'\b', search_space, flags=re.IGNORECASE)
-        re_matches = list(re_matches_iter)
-        return re_matches
-
-    def __fuzzy_match(self, word: str, range_start: int, range_end: int) -> List[Tuple[int, int]]:
-        word = self.__clean_text(word)
-        word = self.__normalize_special_chars(word)
-        word = re.sub(' +', ' ', word, flags=re.DOTALL)
-        word = word.strip()
-        word_character_set = set(word)
-        word_hash = MinHash(num_perm=self.HASH_PERM)
-
-        for char in word_character_set:
-            word_hash.update(char.encode('utf8'))
-
-        candidates = self.hashes.query(word_hash)
-        candidates_split = []
-
-        for candidate in candidates:
-            parts = candidate.split('_')
-            candidates_split.append((parts[0], parts[1], parts[2]))
-
-        candidates_split.sort(key=lambda e: int(e[0]))
-        result = []
-
-        for candidate in candidates_split:
-            c_start = int(candidate[0])
-            c_end = int(candidate[1])
-            token = candidate[2]
-
-            ratio = Levenshtein.normalized_similarity(word, token)
-
-            if ratio < self.SCORE_CUTOFF:
-                continue
-
-            if c_start >= range_start and c_end <= range_end:
-                result.append((c_start, c_end))
-
-        return result
-
     def __get_page_size(self, matches_with_reference) -> Tuple[int, int, int]:
         mwr_sorted = sorted(matches_with_reference, key=lambda x: x.source_span.start)
 
@@ -795,11 +658,12 @@ class ProQuo:
 
     def __search_single_word(self, quote, source_text, search_space_start, search_space_end, page_known_pos):
         source_search_space = source_text[search_space_start:search_space_end]
-        re_matches = self.__strict_match(quote.text, source_search_space)
+        re_matches = Helper.strict_match(quote.text, source_search_space)
         strict_matches_count = len(re_matches)
 
         if strict_matches_count == 0:
-            fuzzy_candidates = self.__fuzzy_match(quote.text, search_space_start, search_space_end)
+            fuzzy_candidates = Helper.fuzzy_match(quote.text, search_space_start, search_space_end, self.hashes,
+                                                  self.SCORE_CUTOFF)
 
             if len(fuzzy_candidates) == 1:
                 return fuzzy_candidates[0][0], fuzzy_candidates[0][1]
@@ -845,7 +709,7 @@ class ProQuo:
 
     def __search_multi_word(self, short_matches, sq, range_start, range_end, source_text):
         search_space = source_text[range_start:range_end]
-        re_matches = self.__strict_match(sq.quote.text, search_space)
+        re_matches = Helper.strict_match(sq.quote.text, search_space)
 
         if len(re_matches) > 1:
             return -1, -1
@@ -853,7 +717,7 @@ class ProQuo:
         if len(re_matches) == 1:
             return re_matches[0].start() + range_start, re_matches[0].end() + range_start
         elif len(re_matches) == 0:
-            re_matches = self.__strict_match(sq.quote.text, source_text)
+            re_matches = Helper.strict_match(sq.quote.text, source_text)
             if len(re_matches) > 1:
                 return -1, -1
 
