@@ -1,5 +1,5 @@
 from os.path import isfile
-from proquo.model.reference.ReferenceModel import ReferenceModel
+from proquo.model.reference.ReferenceModelTrainer import ReferenceModelTrainer
 
 
 def train(train_file_path, val_file_path, output_folder_path):
@@ -42,5 +42,5 @@ def train(train_file_path, val_file_path, output_folder_path):
             else:
                 print(f'wrong count: {line}')
 
-    model = ReferenceModel(25, True, 32, 32, 0.2, 512, 10)
+    model = ReferenceModelTrainer(25, True, 32, 32, 0.2, 512, 10)
     model.train_model(x_train, y_train, x_val, y_val, output_folder_path)
